using System.ComponentModel;

using Geekeey.Extensions.Process.Buffered;
using Geekeey.Extensions.Process.Tests.Dummy;

namespace Geekeey.Extensions.Process.Tests;

[TestFixture]
internal sealed class ExecuteTests
{
	[Test]
	public async Task Execute_ExitCode_And_RunTime()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo"]);

		// Act
		var result = await cmd.ExecuteAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(result.ExitCode, Is.EqualTo(0));
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.RunTime, Is.GreaterThan(TimeSpan.Zero));
		});
	}

	[Test]
	public async Task Execute_ProcessId()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo"]);

		// Act
		var task = cmd.ExecuteAsync();

		// Assert
		Assert.That(task.ProcessId, Is.Not.EqualTo(0));

		await task;
	}

	[Test]
	public async Task Execute_WithAwaiter()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo"]);

		// Act + Assert
		await cmd.ExecuteAsync().ConfigureAwait(false);
	}

	[Test]
	public Task Execute_Throw_On_FileNotFound()
	{
		// Arrange
		var cmd = new Command("some_exe_with_does_not_exits");

		// Act + Assert
		Assert.ThrowsAsync<Win32Exception>(async () => await cmd.ExecuteAsync());
		return Task.CompletedTask;
	}

	[Test]
	public async Task Execute_WithWorkingDirectory()
	{
		// Arrange
		using var dir = TestTempDirectory.Create();

		var cmd = new Command(Program.FilePath)
			.WithArguments("cwd")
			.WithWorkingDirectory(dir.Path);

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		var lines = result.StandardOutput.Split(Environment.NewLine);
		Assert.That(lines, Is.SupersetOf(new[] { dir.Path }));
	}

	[Test]
	public async Task Execute_WithEnvironmentVariables()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["env", "foo", "bar"])
			.WithEnvironmentVariables(env => env
				.Set("foo", "hello")
				.Set("bar", "world"));

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		var lines = result.StandardOutput.Split(Environment.NewLine);
		Assert.That(lines, Is.SupersetOf(new[] { "hello", "world" }));
	}

	[Test]
	public async Task Execute_WithEnvironmentVariables_Overrides()
	{
		// Arrange
		var key = Guid.NewGuid();
		var variableToKeep = $"GKY_TEST_KEEP_{key}";
		var variableToOverwrite = $"GKY_TEST_OVERWRITE_{key}";
		var variableToUnset = $"GKY_TEST_UNSET_{key}";


		using var _a = (TestEnvironment.Create(variableToKeep, "keep")); // will be left unchanged
		using var _b = (TestEnvironment.Create(variableToOverwrite, "overwrite")); // will be overwritten
		using var _c = (TestEnvironment.Create(variableToUnset, "unset")); // will be unset

		var cmd = new Command(Program.FilePath)
			.WithArguments(["env", variableToKeep, variableToOverwrite, variableToUnset])
			.WithEnvironmentVariables(env => env
				.Set(variableToOverwrite, "overwritten")
				.Set(variableToUnset, null));

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		var lines = result.StandardOutput.Split(Environment.NewLine);
		Assert.That(lines, Is.SupersetOf(new[] { "keep", "overwritten" }));
	}
}